"""
Test the manage functions from the CLI
"""

from importlib import reload
import unittest
from unittest.mock import patch

import os
import io
import shutil
import contextlib

from click.testing import CliRunner
import asimov
from asimov.event import Production
from asimov.cli.application import apply_page
from asimov.cli import manage, project
from asimov.ledger import YAMLLedger
from asimov.pipeline import PipelineException

pipelines = {"bayeswave"}
EVENTS = ["GW150914_095045", "GW190924_021846", "GW190929_012149", "GW191109_010717"]


class TestBuild(unittest.TestCase):
    """
    Test the build process.
    """

    @classmethod
    def setUpClass(cls):
        cls.cwd = os.getcwd()

    def tearDown(self):
        os.chdir(self.cwd)
        shutil.rmtree(f"{self.cwd}/tests/tmp/")
    
    def setUp(self):
        os.makedirs(f"{self.cwd}/tests/tmp/project")
        os.chdir(f"{self.cwd}/tests/tmp/project")
        runner = CliRunner()
        result = runner.invoke(project.init,
                               ['Test Project', '--root', f"{self.cwd}/tests/tmp/project"])
        assert result.exit_code == 0
        assert result.output == '● New project created successfully!\n'
        self.ledger = YAMLLedger("ledger.yml")

        f = io.StringIO()
        with contextlib.redirect_stdout(f):
            apply_page(file = "https://git.ligo.org/asimov/data/-/raw/main/defaults/production-pe.yaml", event=None, ledger=self.ledger)
            apply_page(file = "https://git.ligo.org/asimov/data/-/raw/main/defaults/production-pe-priors.yaml", event=None, ledger=self.ledger)
            for event in EVENTS:
                for pipeline in pipelines:
                    apply_page(file = f"https://git.ligo.org/asimov/data/-/raw/main/tests/{event}.yaml", event=None, ledger=self.ledger)
                    apply_page(file = f"https://git.ligo.org/asimov/data/-/raw/main/tests/{pipeline}.yaml", event=event, ledger=self.ledger)

    def test_build_all_events(self):
        """Check that multiple events can be built at once"""
        with patch("asimov.current_ledger", new=YAMLLedger("ledger.yml")):
            reload(asimov)
            reload(manage)
            runner = CliRunner()

            result = runner.invoke(manage.manage, ['build'])
            for event in EVENTS:
                self.assertTrue(f"Working on {event}" in result.output)
                self.assertTrue(f"Production config Prod0 created" in result.output)
                self.assertFalse(f"Production config Prod1 created" in result.output)
                self.assertTrue(os.path.exists(os.path.join(self.cwd, "tests", "tmp", "project", "checkouts", event, "C01_offline", "Prod0.ini")))
                    

    def test_build_dryruns(self):
        """Check that multiple events can be built at once"""
        with patch("asimov.current_ledger", new=YAMLLedger("ledger.yml")):
            reload(asimov)
            reload(manage)
            runner = CliRunner()

            result = runner.invoke(manage.manage, ['build', '--dryrun'])
            for event in EVENTS:
                    self.assertTrue(f"Working on {event}" in result.output)
                    self.assertTrue(f"Will create Prod0" in result.output)

    def test_check_running_events_ignored(self):
        """Check that multiple events can be built at once"""
        with patch("asimov.current_ledger", new=YAMLLedger("ledger.yml")):
            reload(asimov)
            reload(manage)
            runner = CliRunner()
            event = EVENTS[0]
            with open(os.path.join(self.cwd, "tests", "tmp", "project", "test_ledger_page.yaml"), "w") as ledger_page:
                ledger_page.write(f"""
kind: analysis
pipeline: bilby
event: {event}
name: Prod8
status: running
                """)
            apply_page(os.path.join(self.cwd, "tests", "tmp", "project", "test_ledger_page.yaml"), event=event, ledger=self.ledger)

            result = runner.invoke(manage.manage, ['build', '--dryrun'])
            self.assertTrue(f"Working on {event}" in result.output)
            self.assertFalse(f"Will create Prod8" in result.output)


class TestSubmit(unittest.TestCase):
    """
    Test the submit process.
    """

    @classmethod
    def setUpClass(cls):
        cls.cwd = os.getcwd()

    def tearDown(self):
        os.chdir(self.cwd)
        shutil.rmtree(f"{self.cwd}/tests/tmp/")
    
    def setUp(self):
        os.makedirs(f"{self.cwd}/tests/tmp/project")
        os.chdir(f"{self.cwd}/tests/tmp/project")
        runner = CliRunner()
        result = runner.invoke(project.init,
                               ['Test Project', '--root', f"{self.cwd}/tests/tmp/project"])
        assert result.exit_code == 0
        assert result.output == '● New project created successfully!\n'
        self.ledger = YAMLLedger("ledger.yml")

        f = io.StringIO()
        with contextlib.redirect_stdout(f):
            apply_page(file = "https://git.ligo.org/asimov/data/-/raw/main/defaults/production-pe.yaml", event=None, ledger=self.ledger)
            apply_page(file = "https://git.ligo.org/asimov/data/-/raw/main/defaults/production-pe-priors.yaml", event=None, ledger=self.ledger)
            for event in EVENTS:
                for pipeline in pipelines:
                    apply_page(file = f"https://git.ligo.org/asimov/data/-/raw/main/tests/{event}.yaml", event=None, ledger=self.ledger)
                    apply_page(file = f"https://git.ligo.org/asimov/data/-/raw/main/tests/{pipeline}.yaml", event=event, ledger=self.ledger)

    def test_buildsubmit_all_events(self):
        """Check that multiple events can be built at once"""
        with patch("asimov.current_ledger", new=YAMLLedger("ledger.yml")):
            reload(asimov)
            reload(manage)
            runner = CliRunner()

            result = runner.invoke(manage.manage, ['build', 'submit'])
            for event in EVENTS:
                self.assertTrue(f"Working on {event}" in result.output)
                self.assertTrue(f"Production config Prod0 created" in result.output)
                self.assertFalse(f"Production config Prod1 created" in result.output)
                self.assertTrue(os.path.exists(os.path.join(self.cwd, "tests", "tmp", "project", "checkouts", event, "C01_offline", "Prod0.ini")))
                    

    def test_build_submit_dryruns(self):
        """Check that multiple events can be built at once"""
        with patch("asimov.current_ledger", new=YAMLLedger("ledger.yml")):
            reload(asimov)
            reload(manage)
            runner = CliRunner()

            result = runner.invoke(manage.manage, ['build', 'submit', '--dryrun'])
            for event in EVENTS:
                    output = """bayeswave_pipe --trigger-time=1126259462.391 -r """
                    self.assertTrue(output in result.output)

    def test_submit_no_build(self):
        """Check that the command fails as expected if the build has not been completed."""
        runner = CliRunner()
        result = runner.invoke(manage.manage, ['submit', '--dryrun'])
        self.assertTrue("as it hasn't been built yet" in result.output)
                    
    @unittest.skip("I can't get the mocking to work properly.")
    def test_submit_reset(self):
        """Check that an event analysis can be reset"""

        event = EVENTS[0]
        with open(os.path.join(self.cwd, "tests", "tmp", "project", "test_ledger_page.yaml"), "w") as ledger_page:
            ledger_page.write(f"""
kind: analysis
pipeline: bilby
event: {event}
name: Prod8
status: restart
                """)
        apply_page(os.path.join(self.cwd, "tests", "tmp", "project", "test_ledger_page.yaml"), event=event, ledger=self.ledger)

        with patch("asimov.current_ledger", new=YAMLLedger("ledger.yml")):
            reload(asimov)
            reload(manage)
            
            runner = CliRunner()
            
            result = runner.invoke(manage.manage, ['submit', '--dryrun'])
            self.assertTrue(f"Resubmitted {event}/Prod8" in result.output)

    @unittest.skip("I can't get the mocking to work properly.")
    def test_submit_failure(self):
        """Check that failed submits are caught"""
        with patch("asimov.current_ledger", new=YAMLLedger("ledger.yml")):
            reload(asimov)
            reload(manage)
            runner = CliRunner()

            with patch("asimov.event.Production", {"pipeline.submit_dag.side_effect": PipelineException}):
                result = runner.invoke(manage.manage, ['build', 'submit'])
                print(result.output)
                self.assertTrue(f"Unable to submit" in result.output)
            




from supervisor import gitlab, mattermost
from supervisor import config
from supervisor import condor, git
from supervisor.ini import RunConfiguration
import re, os
import shutil, pathlib


def get_psds_rundir(rundir):
    psds = {}
    dets = ['L1', 'H1', 'V1']
    for det in dets:
        asset = f"{rundir}/ROQdata/0/BayesWave_PSD_{det}/post/clean/glitch_median_PSD_forLI_{det}.dat"
        if os.path.exists(asset):
            psds[det] = asset
    return psds



# First, just check how many productions each event has, and build-up from there.

uber_repository = git.MetaRepository("/home/daniel.williams/events/O3/o3a_catalog_events")
server = gitlab.gitlab.Gitlab('https://git.ligo.org', private_token=config.get("gitlab", "token"))
repository = server.projects.get(3816)

events = gitlab.find_events(repository, milestone="PE: C01 Reruns")




for event in events:
    # if event.title in ["S190413ac", "S190503bf", "S190513bm",
    #                    "S190514n", "S190701ah", "S190720", "S190727h", 
    #                    "S190828l", "S190910s", "S190924h", "S190929d"]:
    #     continue

    if "Special" in event.labels:
        continue
    print(event.title)

    try:
        repo = uber_repository.get_repo(event.title)
    except:
        print(f"{event.title} missing from the uberrepo")
        continue

    repo.repo.git.checkout("master")
    repo.repo.git.pull()

    try:
        inis = repo.find_prods()
    except:
        continue

    #for ini in inis:
    #    if "nospin" in ini or "nonspin" in : continue

    nospin = None
    for prod in inis:
        configuration = RunConfiguration(prod)
        if configuration.check_fakecache():
            copy_prod = prod
        else:
            copy_prod = [ini for ini in inis if "Prod0" in ini][0]
        try:
            configuration.ini.get("engine", "disable-spin")
            nospin = True
        except:
            nospin = False
    if not nospin:
        print(event.title, copy_prod)
    else: 
        print(event.title, " This event already has a corrected production.")
        continue

    prods = {ini: int(re.search("Prod([\d]+)", str(ini.split("_"))).groups()[0]) for ini in inis}

    new_prod = (max(prods.values()))+1

    # I know this isn't always using Prod0 any more, cut me some slack, it's been a hectic month, future Daniel.
    prod0_ini = RunConfiguration(copy_prod)
    run_dir = event.data['Prod{}_rundir'.format(prods[copy_prod])]
    print(run_dir)
    psds = get_psds_rundir(run_dir)

    print(list(psds.keys()), prod0_ini.get_ifos())
    

    if set(psds.keys()) == set(prod0_ini.get_ifos()):
        #rint("This is ready to have an ini made")
        pass
    else:
        print("It looks liks some PSDs are still missing.")

    new_approximant = "IMRPhenomD"

    new_ini_loc = os.path.join(pathlib.Path(prod0_ini.ini_loc).parent.absolute(), f"Prod{new_prod}_IMRPhenomD_nospin.ini")
    new_ini = prod0_ini
    new_ini.ini_loc = new_ini_loc

    new_ini.update_webdir(event.title, rootdir = f"public_html/LVC/projects/O3/")
    new_ini.set_approximant("IMRPhenomD", amporder=prod0_ini.ini.get("engine", "amporder"),
                            fref=prod0_ini.ini.get("engine", "fref"))
    new_ini.update_psds(psds)
    new_ini.update_accounting()

    new_ini.ini.set("engine", "disable-spin", "")

    new_ini.save()

    repo.repo.git.add(new_ini_loc)
    repo.repo.git.commit(m=":robot: Added the new C01 Non-spinning config file.")
    repo.repo.git.push()
    

    #print(f"| C01 Prod{new_prod} | {new_approximant} | Offline C01 {



"""Perform tests on the base asimov package."""

import unittest
from unittest.mock import patch
from importlib import reload
import asimov

from pkg_resources import DistributionNotFound

class TestAsimovBase(unittest.TestCase):

    @patch("pkg_resources.get_distribution",
           **{
               'side_effect': DistributionNotFound,#("Not found", "asimov"),
           })
    def testImports(self, blah):
        reload(asimov)

        self.assertEqual(asimov.__version__, "dev")



import os
import shutil
import unittest

import click

import asimov.event
from asimov.ledger import YAMLLedger
from asimov.cli.project import make_project
from asimov.cli.application import apply_page
import git


TEST_LEDGER = """

"""

class DAGTests(unittest.TestCase):
    """All the tests to check production DAGs are generated successfully."""
    @classmethod
    def setUpClass(cls):
        cls.cwd = os.getcwd()
    
    @classmethod
    def tearDownClass(cls):
        """Destroy all the products of this test."""
        os.chdir(cls.cwd)

    def setUp(self):
        os.makedirs(f"{self.cwd}/tests/tmp/project")
        os.chdir(f"{self.cwd}/tests/tmp/project")
        make_project(name="Test project", root=f"{self.cwd}/tests/tmp/project")
        self.ledger = YAMLLedger(f"ledger.yml")
        apply_page(file = "https://git.ligo.org/asimov/data/-/raw/main/defaults/production-pe.yaml", event=None, ledger=self.ledger)
        apply_page(file = "https://git.ligo.org/asimov/data/-/raw/main/events/gwtc-2-1/GW150914_095045.yaml", event=None, ledger=self.ledger)
        
    def tearDown(self):
        shutil.rmtree(f"{self.cwd}/tests/tmp/project")
    
    def test_simple_dag(self):
        """Check that all jobs are run when there are no dependencies specified."""
        apply_page(file = f"{self.cwd}/tests/test_data/test_simple_dag.yaml", event='GW150914_095045', ledger=self.ledger)
        event = self.ledger.get_event('GW150914_095045')[0]
        self.assertEqual(len(event.get_all_latest()), 2)
    
    def test_linear_dag(self):
        """Check that all jobs are run when the dependencies are a chain."""
        apply_page(file = f"{self.cwd}/tests/test_data/test_linear_dag.yaml", event='GW150914_095045', ledger=self.ledger)
        event = self.ledger.get_event('GW150914_095045')[0]
        self.assertEqual(len(event.get_all_latest()), 1)
        

    def test_complex_dag(self):
        """Check that all jobs are run when the dependencies are not a chain."""

        apply_page(file = f"{self.cwd}/tests/test_data/test_complex_dag.yaml", event='GW150914_095045', ledger=self.ledger)
        event = self.ledger.get_event('GW150914_095045')[0]

        self.assertEqual(len(event.get_all_latest()), 2)



import ast
import subprocess
from shutil import copy
from asimov import gitlab, mattermost
from asimov import config
from asimov import condor, git
from asimov.ini import RunConfiguration

import os, glob, datetime

from subprocess import check_output as bash


import otter

server = gitlab.gitlab.Gitlab('https://git.ligo.org', private_token=config.get("gitlab", "token"))
repository = server.projects.get(config.get("olivaw", "tracking_repository"))


uber_repository = git.MetaRepository(config.get("olivaw", "metarepository"))

events = gitlab.find_events(repository, milestone=config.get("olivaw", "milestone"))

mattermost = mattermost.Mattermost()
from pesummary.gw.file.read import read


def get_all_jobs():

    all_jobs = bash('condor_q -af:j Iwd'.split(' '), universal_newlines=True)
    #print(all_jobs)
    all_jobs = str(all_jobs).split('\n')
    #print(all_jobs)
    all_jobs = filter( lambda x: x, all_jobs)

    wds = []
    ids = []
    for z in all_jobs:
        wds.append(z.split()[1])
        ids.append(z.split()[0])
    return ids, wds

def main():

    report = otter.Otter(filename="/home/daniel.williams/public_html/LVC/projects/O3/C01/audit_mcmc.html", 
                         author="R. Daniel Williams",
                         title="Asimov/Olivaw : Preferred run audit report"
    )

    with report:
        report + "This report contains the latest updates on the run status of the various PE runs ongoing at CIT."
        report + "Supervisor last checked these events at " + str(datetime.datetime.now())

    all_ids, all_wds = get_all_jobs()

    for event in events:
        print(event.title)

        if event.state == None:
            continue
        if "Special" in event.labels:
           continue
        try:
            repo = uber_repository.get_repo(event.title)
        except:
            print(f"{event.title} missing from the uberrepo")
            continue

        repo.update(stash=True)

        with report:
            report + f"#{event.title}"

        try:
            event_prods = repo.find_prods(config.get("olivaw", "run_category"))
        except:
            print(f"No runs in this repository")
            continue

        if event.state in ["Generating PSDs", "Productions running", "Stuck"]:
            psds_dict = {}
            prod_keys = [key for key in event.data.keys() if "Prod" in key[0:5]]
            n_productions = len(event_prods)
            event_psds_l = []

            pref_prod = []
            for prod in event_prods:
                prod = prod.split("_")[0]
                if prod in event.data:
                    if "blocked" in event.data[prod].lower(): continue
                    cluster = event.data[prod]
                    prod_rundir = event.data[f"{prod}_rundir"]
                    run_ini = os.path.join(prod_rundir, "config.ini")
                    actual_config = RunConfiguration(run_ini)
                    try:
                        engine_data = actual_config.get_engine()
                    except:
                        continue

                    if not "daniel.williams" in prod_rundir: 
                        continue

                    if actual_config.ini.get("analysis", "engine") == "lalinferencemcmc":
                        # Keep only lists that correspond to the working directory 
                        job_ids = [ number for number, directory in zip(all_ids, all_wds) if (prod in directory) and (event.title in directory) ]
                        print(job_ids)
                        if len(job_ids) > 0:
                            report += job_ids
                        tmp = "tmp"
                        try:
                            os.makedirs(tmp)
                        except:
                            pass

                        try:
                            os.makedirs(f"{prod_rundir}/{tmp}/html")
                            #os.popen(f"rm -r /home/john.veitch/projects/O3/SEOBNRv4P_rota_runs/{event.title}/{prod}")
                            os.makedirs(f"/home/john.veitch/projects/O3/SEOBNRv4P_rota_runs/{event.title}/{prod}-robot")
                        except:
                            pass

                        raw_pp_str = os.popen(f'grep cbcBayesPostProc {prod_rundir}/lalinference*.sh').read()
                        pspath0 = raw_pp_str.split('hdf5_snr.txt ')[-1].split(' ')[0]
                        for job in job_ids:
                            os.system(f'condor_ssh_to_job -ssh scp {job} remote:./*.hdf* {prod_rundir}/{tmp}')
                        
                        for h5file in glob.glob(f"{prod_rundir}/{tmp}/*.hdf5"):
                            pspath1 = h5file # os.path.join(prod_rundir, tmp,'*.hdf5')
                            # print(pspath1)
                            file = h5file.split("/")[-1]
                            copy(h5file, f"/home/john.veitch/projects/O3/SEOBNRv4P_rota_runs/{event.title}/{prod}-robot/{file}")
                            pspath = raw_pp_str.replace(pspath0,pspath1)
                            webpath = pspath.split("--outpath")[1].split()[0]
                        
                        
                            new_webpath = f"{prod_rundir}/{tmp}/html"
                            print(pspath.replace(webpath, new_webpath))
                            # os.popen(pspath.replace(webpath, new_webpath))

main()



"""
Tools for adding data from JSON and YAML files.
Inspired by the kubectl apply approach from kubernetes.
"""

import click
import requests
import yaml

from asimov import LOGGER_LEVEL, logger
import asimov.event
from asimov import current_ledger as ledger
from asimov.utils import update

logger = logger.getChild("cli").getChild("apply")
logger.setLevel(LOGGER_LEVEL)


def apply_page(file, event, ledger=ledger):
    if file[:4] == "http":
        r = requests.get(file)
        if r.status_code == 200:
            data = r.text
            logger.info(f"Downloaded {file}")
        else:
            raise ValueError(f"Could not download this file: {file}")
    else:
        with open(file, "r") as apply_file:
            data = apply_file.read()

    quick_parse = yaml.safe_load_all(
        data
    )  # Load as a dictionary so we can identify the object type it contains

    for document in quick_parse:

        if document["kind"] == "event":
            logger.info("Found an event")
            document.pop("kind")
            event = asimov.event.Event.from_yaml(yaml.dump(document))
            ledger.update_event(event)
            click.echo(
                click.style("●", fg="green") + f" Successfully applied {event.name}"
            )
            logger.info(f"Added {event.name} to project")

        elif document["kind"] == "analysis":
            logger.info("Found an analysis")
            document.pop("kind")
            if event:
                event_s = event
            else:
                if "event" in document:
                    event_s = document["event"]
                else:
                    prompt = "Which event should these be applied to?"
                    event_s = str(click.prompt(prompt))
            try:
                event_o = ledger.get_event(event_s)[0]
            except KeyError as e:
                click.echo(
                    click.style("●", fg="red")
                    + f" Could not apply a production, couldn't find the event {event}"
                )
                logger.exception(e)
            production = asimov.event.Production.from_dict(document, event=event_o)
            try:
                event_o.add_production(production)
                ledger.update_event(event_o)
                click.echo(
                    click.style("●", fg="green")
                    + f" Successfully applied {production.name} to {event_o.name}"
                )
                logger.info(f"Added {production.name} to {event_o.name}")
            except ValueError as e:
                click.echo(
                    click.style("●", fg="red")
                    + f" Could not apply {production.name} to {event_o.name} as "
                    + "an analysis already exists with this name"
                )
                logger.exception(e)

        elif document["kind"] == "configuration":
            logger.info("Found configurations")
            document.pop("kind")
            update(ledger.data, document)
            ledger.save()
            click.echo(
                click.style("●", fg="green")
                + " Successfully applied a configuration update"
            )


@click.command()
@click.option("--file", "-f", help="Location of the file containing the ledger items.")
@click.option(
    "--event",
    "-e",
    help="The event which the ledger items should be applied to (e.g. for analyses)",
    default=None,
)
def apply(file, event):
    apply_page(file, event)



"""
Test that an O3a final event is pushed-through all of the steps of
ini and DAG creation.
"""

import unittest
import os

import asimov.event
from asimov.pipelines import known_pipelines



class EventTests(unittest.TestCase):
    """All tests of the YAML event format."""
    @classmethod
    def setUpClass(cls):
        pass

    def setUp(self):
        with open("GW190426190642.yaml", "r") as f:
            TEST_YAML = f.read()
        TEST_YAML = TEST_YAML.format(wd=os.getcwd())
        self.event = asimov.event.Event.from_yaml(TEST_YAML, repo=False)

    def test_ini_generation(self):
        """Check that the ini file is correctly generated."""

        for production in self.event.productions:
            production.make_config(f"{self.event.name}_{production.name}.ini")

    def test_commandline(self):
        """Check that the correct commandline is run."""
        for production in self.event.productions:
            production.make_config(f"{production.name}.ini")
            if production.pipeline.lower() in known_pipelines:
                try:
                    pipe = known_pipelines[production.pipeline.lower()](production, "C01_offline")
                    pipe.clean()
                    pipe.build_dag()
                except Exception as e:
                    print(production.name)
                    print(e)

if __name__ == '__main__':
    unittest.main()



import glob
import os
import pathlib
import shutil
import subprocess
import time

import git

from copy import copy

from asimov import config, logger
from asimov.utils import set_directory

from .ini import RunConfiguration


class AsimovFileNotFound(FileNotFoundError):
    pass


class EventRepo:
    """
    Read a git repository containing event PE information.

    Parameters
    ----------
    directory : str
       The path to the git repository on the filesystem.
    url : str
       The URL of the git repository
    update : bool
        Flag to determine if the repository is updated when loaded.
        Defaults to False.
    """

    def __init__(self, directory, url=None, update=False):
        self.event = directory.split("/")[-1]
        self.directory = directory
        self.update_needed = update
        self.repo = git.Repo(directory)
        self.url = url

        self.logger = logger

    def __repr__(self):
        return self.directory

    @classmethod
    def create(cls, location):
        """
        Create a new git repository to store configurations etc.

        Parameters
        ----------
        location : str
           The location of the directory to be used.
        """
        directory = config.get("general", "calibration_directory")
        os.makedirs(location, exist_ok=True)
        repo = git.Repo.init(location)
        os.makedirs(os.path.join(location, directory), exist_ok=True)
        with open(os.path.join(location, directory, ".gitkeep"), "w") as f:
            f.write(" ")
        repo.git.add(os.path.join(".", directory, ".gitkeep"))
        try:
            repo.git.commit("-m", "Initial commit")
        except git.exc.GitCommandError as e:
            if "working tree clean" in e.stdout:
                pass
        return cls(directory=location, url=location)

    @classmethod
    def from_url(cls, url, name, directory=None, update=False):
        """
        Clone a git repository into a working directory,
        then create an EventRepo object for it.

        Parameters
        ----------
        url : str
           The URL of the git repository
        name : str
           The name for the git repository (probably the event name)
        directory : str, optional
           The location to store the cloned repository.
           If this value isn't provided the repository is
           cloned into the /tmp directory.
        update : bool
           Flag to determine if the repository is updated when loaded.
           Defaults to False.
        """
        if not directory:
            tmp = config.get("general", "git_default")
            directory = f"{tmp}/{name}"

            if os.path.exists(directory):
                return cls(directory, url, update=update)

            pathlib.Path(directory).mkdir(parents=True, exist_ok=True)

        # Replace an https address with an ssh address
        if "https" in url:
            url = url.replace("https://", "git@")
            final = "/".join(url.split("/")[1:])
            start = url.split("/")[0]

            url = f"{start}:{final}"

        try:
            repo = git.Repo.clone_from(url, directory)
            repo.git.execute(["git", "lfs", "install"])
            repo.git.execute(["git", "lfs", "fetch"])
            repo.git.execute(["git", "lfs", "pull"])
        except git.exc.GitCommandError:
            repo = git.Repo(directory)
            try:
                repo.git.stash()
            except git.exc.GitCommandError:
                pass

        if update:
            try:
                repo.remotes[0].pull()
            except git.exc.GitCommandError:
                pass
        return cls(directory, url, update=update)

    def add_file(self, source, destination, commit_message=None):
        """
        Add a new file to the repository.

        Parameters
        ----------
        source : str, file path
           The path to the file to be added.
        destination : str
           The location to which the file should be copied in
           the repository, relative to the root of the repository.
           Any directories which do not exist already will be created.
        commit_message : str, optional
           The commit message for the git commit.
           Defaults to a description of the file addition.
        """

        destination_dir = os.path.dirname(destination)
        destination_dir = os.path.join(self.directory, destination_dir)
        pathlib.Path(destination_dir).mkdir(parents=True, exist_ok=True)

        destination_d = os.path.join(self.directory, destination)

        try:
            shutil.copyfile(source, destination_d)
        except shutil.SameFileError:
            pass

        if not commit_message:
            commit_message = f"Added {destination}"

        self.repo.git.add(destination)
        self.repo.git.commit("-m", commit_message)
        try:
            self.repo.git.push()
            time.sleep(15)
        except git.exc.GitCommandError as e:
            if "There is no tracking information for the current branch." in str(e):
                pass
            elif (
                "Either specify the URL from the command-line or configure a remote repository using"
                in str(e)
            ):
                pass
            else:
                raise e

    def find_timefile(self, category=config.get("general", "calibration_directory")):
        """
        Find the time file in this repository.
        """

        with set_directory(os.path.join(self.directory, category)):
            try:
                gps_file = glob.glob("*gps*.txt")[0]
                return gps_file
            except IndexError:
                raise AsimovFileNotFound

    def find_coincfile(self, category=config.get("general", "calibration_directory")):
        """
        Find the coinc file for this calibration category in this repository.
        """
        coinc_file = glob.glob(os.path.join(self.directory, category, "*coinc*.xml"))

        if len(coinc_file) > 0:
            return coinc_file[0]
        else:
            raise AsimovFileNotFound

    def find_prods(
        self, name=None, category=config.get("general", "calibration_directory")
    ):
        """
        Find all of the productions for a relevant category of runs
        in the event repository.

        Parameters
        ----------
        name : str, optional
           The name of the production.
           If omitted then all production ini files are returned.
        category : str, optional
           The category of run. Defaults to "general/calibration_directory" from the config file.
        """

        self.update()
        path = f"{os.path.join(os.getcwd(), self.directory, category)}/{name}.ini"
        return [path]

    def upload_prod(
        self,
        production,
        rundir,
        preferred=False,
        category=config.get("general", "calibration_directory"),
        rootdir="public_html/LVC/projects/O3/C01/",
        rename=False,
    ):
        """
        Upload the results of a PE job to the event repostory.

        Parameters
        ----------
        category : str, optional
           The category of the job.
           Defaults to "C01_offline".
        production : str
           The production name.
        rundir : str
           The run directory of the PE job.
        """

        preferred_list = ["--preferred", "--append_preferred"]
        web_path = os.path.join(
            os.path.expanduser("~"), *rootdir.split("/"), self.event, production
        )  # TODO Make this generic
        if rename:
            prod_name = rename
        else:
            prod_name = production

        command = [
            config.get("pesummary", "location"),
            "--event",
            self.event,
            "--exp",
            prod_name,
            "--rundir",
            rundir,
            "--webdir",
            web_path,
            "--edit_homepage_table",
        ]
        if preferred:
            command += preferred_list
        dagman = subprocess.Popen(
            command, stdout=subprocess.PIPE, stderr=subprocess.STDOUT
        )
        out, err = dagman.communicate()

        if err or "master -> master" not in str(out):
            raise ValueError(f"Sample upload failed.\n{out}\n{err}")
        else:
            return out

    def upload_preferred(self, event, prods):
        """
        Prepare the preferred PESummary file by combining all of the
        productions for an event which are marked as `Preferred`
        or `Finalised`.

        Parameters
        ----------
        event : `asimov.gitlab.EventIssue`
           The event which the preferred upload is being prepared for.
        prods : list
           A list of all of the productions which should be included in the preferred file.
        """

        samples = []
        labels = []
        configs = []

        for prod in prods:
            samples.append(
                glob.glob(
                    str(
                        os.path.join(event.data[f"{prod}_rundir"], "posterior_samples"),
                    )
                    + "/*.hdf5"
                )[0]
            )
            run_ini = os.path.join(event.data[f"{prod}_rundir"], "config.ini")
            actual_config = RunConfiguration(run_ini)
            engine_data = actual_config.get_engine()
            labels.append(f"C01:{engine_data['approx']}")
            configs.append(
                str(os.path.join(event.data[f"{prod}_rundir"], "config.ini"))
            )

        with set_directory(
            os.path.join(self.directory, "Preferred", "PESummary_metafile")
        ):

            command = [
                "summarycombine",
                "--webdir",
                f"/home/daniel.williams/public_html/LVC/projects/O3/preferred/{event.title}",
                "--samples",
            ]
            command += samples
            command += ["--labels"]
            command += labels
            command += ["--config"]
            command += configs
            command += ["--gw"]

            dagman = subprocess.Popen(
                command, stdout=subprocess.PIPE, stderr=subprocess.STDOUT
            )
            out, err = dagman.communicate()

            self.logger.info(out)
            self.logger.error(err)

            copy(
                "/home/daniel.williams/public_html/LVC/projects/O3/"
                + f"preferred/{event.title}/samples/posterior_samples.h5",
                os.path.join(self.directory, "Preferred", "PESummary_metafile"),
            )
            self.repo.git.add("Preferred/PESummary_metafile/posterior_samples.h5")
            self.repo.git.commit("-m", "Updated the preferred sample metafile.")
            self.repo.git.push()
            time.sleep(15)

            event.labels += ["Preferred cleaned"]
            event.issue_object.save()

        return True

    def update(self, stash=False, branch="master"):
        """
        Pull the latest updates to the repository.

        Parameters
        ----------
        stash : bool, optional
           If true any changes which are in the local version
           of the repository are first stashed.
           Default is False.
        branch : str, optional
           The branch which should be checked-out.
           Default is master.
        """
        if stash:
            self.repo.git.stash()

        self.repo.git.checkout(branch)
        try:
            self.repo.git.pull()
            self.repo.git.execute(["git", "lfs", "fetch"])
        except git.exc.GitCommandError as e:
            if "There is no tracking information for the current branch." in str(e):
                pass
            elif (
                "Either specify the URL from the command-line or configure a remote repository using"
                in str(e)
            ):
                pass
            elif "Temporary failure in name resolution" in str(e):
                logger.warning(f"Unable to update the repository for {self.event}")
            else:
                raise e



"""
Handle run configuration files.

This module is inteded to read and manipulate the run configuration
files which are used by ``LALInferencePipe`` to design a parameter estimation run.


"""
import ast
import getpass
import os
from configparser import ConfigParser, NoOptionError


class RunConfiguration(object):
    """A class to represent a run configuration."""

    def __init__(self, path):
        """
        Open the run configuration file.

        Parameters
        ----------
        path : str
           The path to a run configuration ini file.
        """
        self.ini_loc = path
        ini = ConfigParser()
        ini.optionxform = str

        if type(path) == dict:
            ini.read_dict(path)
        else:

            try:
                ini.read(path)
            except FileNotFoundError:
                raise ValueError("Could not open the ini file")

        self.ini = ini

    def check_fakecache(self):
        """
        Check to see if this file contains a fake-cache.

        This can be used to determine whether the ini file sets up a configuration which
        uses e.g. deglitched frames.

        Returns
        -------
        bool
           Returns true if a fake cache has been used for this file.
        """
        try:
            if len(self.ini.get("lalinference", "fake-cache")) > 0:
                return True
            else:
                return False
        except NoOptionError:
            return False

    def set_lalinference(self, **kwargs):
        """
        Set the values of LALInference configuration variables.

        Parameters
        ----------
        kwargs :
           Set a variety of parameters.

        """
        for key, value in kwargs:
            self.ini.set("lalinference", key, value)

    def get_psds(self):
        """
        Check if PSDs have been set in the ini file,
        and return them if they have.
        """
        psds = {}
        for det in self.get_ifos():
            try:
                psds[det] = self.ini.get("engine", f"{det}-psd")
            except NoOptionError:
                pass
        return psds

    def get_calibration(self):
        """
        Retrieve the calibration envelope locations.
        """
        calibration = {}
        for det in self.get_ifos():
            try:
                calibration[det] = self.ini.get("engine", f"{det}-spcal-envelope")
            except NoOptionError:
                pass
        return calibration

    def update_psds(self, psds, clobber=False):
        """
        Update the locations of the PSDs in the ini file.

        Parameters
        ----------
        psds : dict
           A dictionary of the various PSDs.
        """
        for det, location in psds.items():
            needs_psd = True
            try:
                self.ini.get("engine", f"{det}-psd")
                needs_psd = False
            except NoOptionError:
                needs_psd = True

            if needs_psd or clobber:
                self.ini.set("engine", f"{det}-psd", location)

    def run_bayeswave(self, status=True):
        """
        Ensure that Bayeswave is run for this job.

        This should be run in order to add Bayeswave to produce PSDs for an event.

        Parameters
        ----------
        status : bool, optional
           If set to true (the default) then a line will be added to ensure that
           Bayeswave is run to generate PSDs. If False then this line will be
           removed from the ini file if it exists.
        """
        if status:
            self.ini.set("condor", "bayeswave", "%(lalsuite-install)s/bin/BayesWave")
        else:
            try:
                self.ini.remove_option("condor", "bayeswave")
            except NoOptionError:
                pass

    def get_engine(self):
        """
        Fetch all of the Lalinference engine data.
        """
        return dict(self.ini.items("engine"))

    def get_ifos(self):
        return ast.literal_eval(self.ini.get("analysis", "ifos"))

    def get_channels(self):
        return ast.literal_eval(self.ini.get("data", "channels"))

    def set_approximant(self, approximant, amporder, fref):
        self.ini.set("engine", "approx", approximant)
        self.ini.set("engine", "amporder", amporder)
        self.ini.set("engine", "fref", fref)

    def set_queue(self, queue="Priority_PE"):
        self.ini.set("condor", "queue", queue)

    def update_accounting(self, user=None):
        """
        Update the accounting tag for this job.
        Defaults to the user account running the supervisor.

        Parameters
        ----------
        user : str
           The accounting user to be added to the ini file.
        """

        if not user:
            user = self._get_user()
        self.ini.set("condor", "accounting_group_user", user)

    def update_webdir(self, event, prod, rootdir="public_html/LVC/projects/O3/C01/"):
        """
        Update the web directory in the ini file.
        """

        web_path = os.path.join(
            os.path.expanduser("~"), *rootdir.split("/"), event, prod
        )  # TODO Make this generic
        self.ini.set("paths", "webdir", web_path)

    def _get_user(self):
        user = getpass.getuser()
        return user

    def save(self):
        with open(self.ini_loc, "w") as fp:
            self.ini.write(fp)



import click

from asimov import config


@click.group()
def configuration():
    """Group for all of the configuration-related command line stuff."""
    pass


@configuration.command()
@click.option("--key", "-k", "key", default=None, help="Show a specific key")
def show(key):
    """Show all configuration values"""
    if key:
        section, key = key.split("/")
        section = section.replace("-", "")
        click.echo(config.get(section, key))
    else:
        for section, values in config._sections.items():
            click.echo(click.style(section, fg="black", bg="white"))
            for key, value in values.items():
                click.secho(f"\t{key:30}\t{value}", bold=False)


@configuration.command(
    context_settings=dict(
        ignore_unknown_options=True,
    )
)
@click.argument("kwargs", nargs=-1, type=click.UNPROCESSED)
def update(kwargs):
    """Update a configuration setting."""
    key, value = kwargs
    section, key = key.split("/")
    section = section.replace("-", "")
    config.set(section, key, value)
    with open("asimov.conf", "w") as config_file:
        config.write(config_file)



import os
import sys

# Ignore warnings from the condor module
import warnings

import click

warnings.filterwarnings("ignore", module="htcondor")  # NoQA

# Replace this with a better logfile handling module please
# from glob import glob
import asimov  # NoQA
import asimov.pipelines  # NoQA

# Import CLI bits from elsewhere
from asimov.cli import (  # NoQA
    application,
    configuration,
    event,
    manage,
    monitor,
    production,
    project,
    report,
    review,
)  # NoQA


@click.version_option(asimov.__version__)
@click.group()
@click.pass_context
def olivaw(ctx):
    """
    This is the main program which runs the DAGs for each event issue.
    """

    # Check that we're running in an actual asimov project
    if not os.path.exists("asimov.conf") and ctx.invoked_subcommand != "init":
        # This isn't the root of an asimov project, let's fail.
        click.secho("This isn't an asimov project", fg="white", bg="red")
        sys.exit(1)
    pass


# Project initialisation
olivaw.add_command(project.init)
olivaw.add_command(project.clone)

olivaw.add_command(event.event)

# Building and submission
olivaw.add_command(manage.manage)
# Reporting commands
olivaw.add_command(report.report)
# Configuration commands
olivaw.add_command(configuration.configuration)
# Monitoring commands
olivaw.add_command(monitor.start)
olivaw.add_command(monitor.stop)
olivaw.add_command(monitor.monitor)
olivaw.add_command(event.event)
olivaw.add_command(production.production)
# Review commands
olivaw.add_command(review.review)
olivaw.add_command(application.apply)



"""Bilby Pipeline specification."""

import glob
import os
import re
import subprocess
import configparser

import git.exc
import time

from liquid import Liquid

from asimov.utils import update

from .. import config
from ..pipeline import Pipeline, PipelineException, PipelineLogger, PESummaryPipeline


class Bilby(Pipeline):
    """
    The Bilby Pipeline.

    Parameters
    ----------
    production : :class:`asimov.Production`
       The production object.
    category : str, optional
        The category of the job.
        Defaults to "C01_offline".
    """

    name = "Bilby"
    STATUS = {"wait", "stuck", "stopped", "running", "finished"}

    def __init__(self, production, category=None):
        super(Bilby, self).__init__(production, category)
        self.logger.info("Using the bilby pipeline")

        if not production.pipeline.lower() == "bilby":
            raise PipelineException

    def detect_completion(self):
        """
        Check for the production of the posterior file to signal that the job has completed.
        """
        self.logger.info("Checking if the bilby job has completed")
        results_dir = glob.glob(f"{self.production.rundir}/result")
        if len(results_dir) > 0:  # dynesty_merge_result.json
            results_files = glob.glob(
                os.path.join(results_dir[0], "*merge*_result.hdf5")
            )
            results_files += glob.glob(
                os.path.join(results_dir[0], "*merge*_result.json")
            )
            self.logger.debug(f"results files {results_files}")
            if len(results_files) > 0:
                self.logger.info("Results files found, the job is finished.")
                return True
            else:
                self.logger.info("No results files found.")
                return False
        else:
            self.logger.info("No results directory found")
            return False

    def before_submit(self):
        """
        Pre-submit hook.
        """
        self.logger.info("Running the before_submit hook")
        sub_files = glob.glob(f"{self.production.rundir}/submit/*.submit")
        for sub_file in sub_files:
            if "dag" in sub_file:
                continue
            with open(sub_file, "r") as f_handle:
                original = f_handle.read()
            with open(sub_file, "w") as f_handle:
                self.logger.info(f"Adding preserve_relative_paths to {sub_file}")
                f_handle.write("preserve_relative_paths = True\n" + original)

    def _determine_prior(self):
        """
        Determine the correct choice of prior file for this production.
        """

        self.logger.info("Determining the prior file for this production")

        if "prior file" in self.production.meta:
            self.logger.info("A prior file has already been specified.")
            self.logger.info(f"{self.production.meta['prior file']}")
            return self.production.meta["prior file"]
        else:
            template = None

            if "event type" in self.production.meta:
                event_type = self.production.meta["event type"].lower()
            else:
                event_type = "bbh"
                self.production.meta["event type"] = event_type

                if self.production.event.issue_object:
                    self.production.event.issue_object.update_data()

            if template is None:
                template_filename = f"{event_type}.prior.template"
                self.logger.info(
                    f"[bilby] Constructing a prior using {event_type}.prior.template."
                )
                try:
                    template = os.path.join(
                        config.get("bilby", "priors"), template_filename
                    )
                except (configparser.NoOptionError, configparser.NoSectionError):
                    from pkg_resources import resource_filename

                    template = resource_filename(
                        "asimov", f"priors/{template_filename}"
                    )

            priors = {}
            priors = update(priors, self.production.event.ledger.data["priors"])
            priors = update(priors, self.production.event.meta["priors"])
            priors = update(priors, self.production.meta["priors"])

            liq = Liquid(template)
            rendered = liq.render(priors=priors, config=config)

            prior_name = f"{self.production.name}.prior"
            prior_file = os.path.join(os.getcwd(), prior_name)
            self.logger.info(f"Saving the new prior file as {prior_file}")
            with open(prior_file, "w") as new_prior:
                new_prior.write(rendered)

            repo = self.production.event.repository
            try:

                repo.add_file(
                    prior_file,
                    os.path.join(
                        config.get("general", "calibration_directory"), prior_name
                    ),
                )
                os.remove(prior_file)
            except git.exc.GitCommandError:
                pass
            return os.path.join(
                self.production.event.repository.directory,
                config.get("general", "calibration_directory"),
                prior_name,
            )

    def build_dag(self, psds=None, user=None, clobber_psd=False, dryrun=False):
        """
        Construct a DAG file in order to submit a production to the
        condor scheduler using bilby_pipe.

        Parameters
        ----------
        production : str
           The production name.
        psds : dict, optional
           The PSDs which should be used for this DAG. If no PSDs are
           provided the PSD files specified in the ini file will be used
           instead.
        user : str
           The user accounting tag which should be used to run the job.
        dryrun: bool
           If set to true the commands will not be run, but will be printed to standard output. Defaults to False.

        Raises
        ------
        PipelineException
           Raised if the construction of the DAG fails.
        """

        cwd = os.getcwd()

        self.logger.info(f"Working in {cwd}")

        self._determine_prior()  # Build the prior file

        if self.production.event.repository:
            ini = self.production.event.repository.find_prods(
                self.production.name, self.category
            )[0]
            ini = os.path.join(cwd, ini)
        else:
            ini = f"{self.production.name}.ini"

        if self.production.rundir:
            rundir = self.production.rundir
        else:
            rundir = os.path.join(
                os.path.expanduser("~"),
                self.production.event.name,
                self.production.name,
            )
            self.production.rundir = rundir

        if "job label" in self.production.meta:
            job_label = self.production.meta["job label"]
        else:
            job_label = self.production.name

        command = [
            os.path.join(config.get("pipelines", "environment"), "bin", "bilby_pipe"),
            ini,
            "--label",
            job_label,
            "--outdir",
            f"{os.path.abspath(self.production.rundir)}",
            "--accounting",
            f"{self.production.meta['scheduler']['accounting group']}",
        ]

        if dryrun:
            print(" ".join(command))
        else:
            self.logger.info(" ".join(command))
            pipe = subprocess.Popen(
                command, stdout=subprocess.PIPE, stderr=subprocess.STDOUT
            )
            out, err = pipe.communicate()
            self.logger.info(out)

            if err or "DAG generation complete, to submit jobs" not in str(out):
                self.production.status = "stuck"
                self.logger.error(err)
                raise PipelineException(
                    f"DAG file could not be created.\n{command}\n{out}\n\n{err}",
                    production=self.production.name,
                )
            else:
                time.sleep(10)
                return PipelineLogger(message=out, production=self.production.name)

    def submit_dag(self, dryrun=False):
        """
        Submit a DAG file to the condor cluster.

        Parameters
        ----------
        dryrun : bool
           If set to true the DAG will not be submitted,
           but all commands will be printed to standard
           output instead. Defaults to False.

        Returns
        -------
        int
           The cluster ID assigned to the running DAG file.
        PipelineLogger
           The pipeline logger message.

        Raises
        ------
        PipelineException
           This will be raised if the pipeline fails to submit the job.

        Notes
        -----
        This overloads the default submission routine, as bilby seems to store
        its DAG files in a different location
        """

        cwd = os.getcwd()
        self.logger.info(f"Working in {cwd}")

        self.before_submit()

        try:
            # to do: Check that this is the correct name of the output DAG file for billby (it
            # probably isn't)
            if "job label" in self.production.meta:
                job_label = self.production.meta["job label"]
            else:
                job_label = self.production.name
            dag_filename = f"dag_{job_label}.submit"
            command = [
                # "ssh", f"{config.get('scheduler', 'server')}",
                "condor_submit_dag",
                "-batch-name",
                f"bilby/{self.production.event.name}/{self.production.name}",
                os.path.join(self.production.rundir, "submit", dag_filename),
            ]

            if dryrun:
                print(" ".join(command))
            else:

                # with set_directory(self.production.rundir):
                self.logger.info(f"Working in {os.getcwd()}")

                dagman = subprocess.Popen(
                    command, stdout=subprocess.PIPE, stderr=subprocess.STDOUT
                )

                self.logger.info(" ".join(command))

                stdout, stderr = dagman.communicate()

                if "submitted to cluster" in str(stdout):
                    cluster = re.search(
                        r"submitted to cluster ([\d]+)", str(stdout)
                    ).groups()[0]
                    self.logger.info(
                        f"Submitted successfully. Running with job id {int(cluster)}"
                    )
                    self.production.status = "running"
                    self.production.job_id = int(cluster)
                    return cluster, PipelineLogger(stdout)
                else:
                    self.logger.error("Could not submit the job to the cluster")
                    self.logger.info(stdout)
                    self.logger.error(stderr)

                    raise PipelineException(
                        "The DAG file could not be submitted.",
                    )

        except FileNotFoundError as error:
            self.logger.exception(error)
            raise PipelineException(
                "It looks like condor isn't installed on this system.\n"
                f"""I wanted to run {" ".join(command)}."""
            ) from error

    def collect_assets(self):
        """
        Gather all of the results assets for this job.
        """
        return {"samples": self.samples()}

    def samples(self, absolute=False):
        """
        Collect the combined samples file for PESummary.
        """

        if absolute:
            rundir = os.path.abspath(self.production.rundir)
        else:
            rundir = self.production.rundir
        self.logger.info(f"Rundir for samples: {rundir}")
        return glob.glob(
            os.path.join(rundir, "result", "*_merge*_result.hdf5")
        ) + glob.glob(os.path.join(rundir, "result", "*_merge*_result.json"))

    def after_completion(self):
        post_pipeline = PESummaryPipeline(production=self.production)
        self.logger.info("Job has completed. Running PE Summary.")
        cluster = post_pipeline.submit_dag()
        self.production.meta["job id"] = int(cluster)
        self.production.status = "processing"
        self.production.event.update_data()

    def collect_logs(self):
        """
        Collect all of the log files which have been produced by this production and
        return their contents as a dictionary.
        """
        logs = glob.glob(f"{self.production.rundir}/submit/*.err") + glob.glob(
            f"{self.production.rundir}/log*/*.err"
        )
        logs += glob.glob(f"{self.production.rundir}/*/*.out")
        messages = {}
        for log in logs:
            try:
                with open(log, "r") as log_f:
                    message = log_f.read()
                    message = message.split("\n")
                    messages[log.split("/")[-1]] = "\n".join(message[-100:])
            except FileNotFoundError:
                messages[
                    log.split("/")[-1]
                ] = "There was a problem opening this log file."
        return messages

    def check_progress(self):
        """
        Check the convergence progress of a job.
        """
        logs = glob.glob(f"{self.production.rundir}/log_data_analysis/*.out")
        messages = {}
        for log in logs:
            try:
                with open(log, "r") as log_f:
                    message = log_f.read()
                    message = message.split("\n")[-1]
                    p = re.compile(r"([\d]+)it")
                    iterations = p.search(message)
                    p = re.compile(r"dlogz:([\d]*\.[\d]*)")
                    dlogz = p.search(message)
                    if iterations:
                        messages[log.split("/")[-1]] = (
                            iterations.group(),
                            dlogz.group(),
                        )
            except FileNotFoundError:
                messages[
                    log.split("/")[-1]
                ] = "There was a problem opening this log file."
        return messages

    @classmethod
    def read_ini(cls, filepath):
        """
        Read and parse a bilby configuration file.

        Note that bilby configurations are property files and not compliant ini configs.

        Parameters
        ----------
        filepath: str
           The path to the ini file.
        """

        with open(filepath, "r") as f:
            file_content = "[root]\n" + f.read()

        config_parser = configparser.RawConfigParser()
        config_parser.read_string(file_content)

        return config_parser

    def html(self):
        """Return the HTML representation of this pipeline."""
        pages_dir = os.path.join(self.production.event.name, self.production.name)
        out = ""
        out += """<div class="asimov-pipeline bilby">"""
        out += """<ul>"""
        out += f"""<li><a href="{pages_dir}/overview.html">Overview pages</a></li>"""
        if self.production.status in {"finished", "uploaded"}:
            if self.production.status == "uploaded":
                out += f"""<li><a href="{pages_dir}/pesummary/home.html">Summary pages</a></li>"""
        out += """</ul>"""

        image_card = """<div class="card" style="width: 18rem;">
<img class="card-img-top" src="{0}" alt="Card image cap">
  <div class="card-body">
    <p class="card-text">{1}</p>
  </div>
</div>
        """

        plots = [
            [
                f"{pages_dir}/pesummary/plots/{self.production.name}_1d_posterior_luminosity_distance.png",
                "luminosity distance",
            ],
            [
                f"{pages_dir}/pesummary/plots/{self.production.name}_skymap.png",
                "skymap",
            ],
        ]

        out += """<div class="card-group">"""
        for plot in plots:
            if self.production.status in {"finished", "uploaded"}:
                out += image_card.format(plot[0], plot[1])

        out += """</div>"""  # Closes card-group
        out += """</div>"""  # Closes the bilby div
        return out

    def resurrect(self):
        """
        Attempt to ressurrect a failed job.
        """
        try:
            count = self.production.meta["resurrections"]
        except KeyError:
            count = 0
        if (count < 5) and (
            len(glob.glob(os.path.join(self.production.rundir, "submit", "*.rescue*")))
            > 0
        ):
            count += 1
            self.submit_dag()



"""
Supervisor: A package for interacting with
long-running data analysis jobs.
"""


__author__ = """Daniel Williams"""
__email__ = "daniel.williams@ligo.org"
__packagename__ = __name__

import os
import logging

from pkg_resources import DistributionNotFound, get_distribution, resource_string

try:
    __version__ = get_distribution(__name__).version
except DistributionNotFound:
    # package is not installed
    __version__ = "dev"
    pass

try:
    import ConfigParser as configparser
except ImportError:
    import configparser
default_config = resource_string(__name__, "{}.conf".format(__packagename__))

config = configparser.ConfigParser()
# if not config_file:

config.read_string(default_config.decode("utf8"))
config_locations = [
    os.path.join(os.curdir, "{}.conf".format(__packagename__)),
    os.path.join(
        os.path.expanduser("~"),
        ".config",
        __packagename__,
        "{}.conf".format(__packagename__),
    ),
    os.path.join(os.path.expanduser("~"), ".{}".format(__packagename__)),
    "/etc/{}".format(__packagename__),
]

config_locations.reverse()

config.read([conffile for conffile in config_locations])


logging.getLogger("werkzeug").setLevel(logging.WARNING)
logging.getLogger("MARKDOWN").setLevel(logging.WARNING)
logging.getLogger("matplotlib").setLevel(logging.WARNING)
logging.getLogger("git").setLevel(logging.WARNING)


logger_name = "asimov"
logger = logging.getLogger(logger_name)

logger_levels = {
    "debug": logging.DEBUG,
    "info": logging.INFO,
    "warning": logging.WARNING,
    "error": logging.ERROR,
    "update": 9,
}
try:
    LOGGER_LEVEL = logger_levels[config.get("logging", "logging level")]
except configparser.NoOptionError:
    LOGGER_LEVEL = logging.INFO

try:
    PRINT_LEVEL = logger_levels[config.get("logging", "print level")]
except configparser.NoOptionError:
    PRINT_LEVEL = logging.ERROR

ch = logging.StreamHandler()
print_formatter = logging.Formatter("[%(levelname)s] %(message)s", "%Y-%m-%d %H:%M:%S")
ch.setFormatter(print_formatter)
ch.setLevel(PRINT_LEVEL)

logfile = "asimov.log"
fh = logging.FileHandler(logfile)
formatter = logging.Formatter(
    "%(asctime)s [%(name)s][%(levelname)s] %(message)s", "%Y-%m-%d %H:%M:%S"
)
fh.setFormatter(formatter)
fh.setLevel(LOGGER_LEVEL)

logger.addHandler(ch)
logger.addHandler(fh)

try:
    if config.get("ledger", "engine") == "gitlab":
        from .gitlab import GitlabLedger

        current_ledger = GitlabLedger()
    elif config.get("ledger", "engine") == "yamlfile":
        from .ledger import YAMLLedger

        current_ledger = YAMLLedger(config.get("ledger", "location"))
    else:
        current_ledger = None
except FileNotFoundError:
    # logger.error("Could not find a valid ledger file.")
    current_ledger = None
